package l14_reflection;

import java.lang.reflect.Method;

public class GetMethodsTest {

	static class X {
		private void m1() {}
		public void m2(int dummy) {}
	}

	public static void main(String[] args) {
		testClass("java.lang.Boolean");
		testClass("l14_reflection.GetMethodsTest$X");

		try { // instance method
			Method method = Integer.class.getMethod("compareTo", Integer.class);
			int ret = (int)method.invoke(25, 30);
			System.out.println(STR."compareTo = \{ret}");
		} catch (Exception e) {
			e.printStackTrace();
		}

		try { // class method
			Method method = Integer.class.getMethod("valueOf", String.class);
			int ret = (int)method.invoke(null, "12");
			System.out.println(STR."valueOf = \{ret}");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static void testClass(String className) {
		Class<?> cls;

		try {
			//	Class.forName simply loads a class, including running its static initializers
			cls = Class.forName(className);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			return;
		}

		//	returns the array of Method objects representing the public methods of this class
		//	(declared and inherited)

		System.out.println(STR."------------ Class \{className} --------------");
		System.out.println("Methods (public; declared and inherited)");

		Method[] methods = cls.getMethods();
		for (Method method : methods) {
			System.out.println(STR."\t\{method}");
		}
		System.out.println();

		if (methods.length > 0) {
			System.out.println(methods[0]);
			Class<?>[] parameterTypes = methods[0].getParameterTypes();
			for (Class<?> parameter : parameterTypes) {
				System.out.println(STR."\tparameter: \{parameter.getName()}");
			}
		}
		System.out.println();

		//	returns the array of Method objects representing declared methods of this class
		//	(not inherited)

		System.out.println("Declared Methods (not inherited)");
		methods = cls.getDeclaredMethods();
		for (Method method : methods) {
			System.out.println(STR."\t\{method}");
		}
		System.out.println();
	}
} 

