package l14_reflection;

//	Instances of the class Class represent classes and interfaces in a running
//	Java application. Every array also belongs to a class that is reflected as
//	a Class object that is shared by all arrays with the same element type and
//	number of dimensions. The primitive Java types (boolean, byte, char, short,
//	int, long, float, and double), and the keyword void are also represented as
//	Class objects.

//	Class has no public constructor. Instead, Class objects are constructed
//	automatically by the Java Virtual Machine as classes are loaded and by calls
//	to the defineClass method in the class loader. 

//	One useful real-world use of reflection is when writing a framework that has
//	to interoperate with user-defined classes, where the framework author doesn't
//	know what the members (or even the classes) will be. Reflection allows them
//	to deal with any class without knowing it in advance.

class Names {

	interface FirstInterface {}
	interface SecondInterface {}

	static class MyClass implements FirstInterface, SecondInterface {}

	private static void printName(String name, String value) {
		System.out.format("%-30s : %s%n", name, value);
	}

	private static void testNames() {
		printName("Object.class.getName()", Object.class.getName());
		printName("Object.class.toString", Object.class.toString());
		printName("MyClass.class.getName()", MyClass.class.getName());
		printName("MyClass.class.getSimpleName()", MyClass.class.getSimpleName());
		System.out.println();

		Class<?>[] interfaces = MyClass.class.getInterfaces();
		printName("MyClass.class.getInterfaces()", "");
		for(Class<?> i : interfaces) {
			printName("", i.getName());
		}
		System.out.println();

		printName("int.class.getName()", int.class.getName());
		printName("int[].class.getName()", int[].class.getName());
		printName("double[][][].class.getName()", double[][][].class.getName());
	}

	public static void main (String[] args) {
		testNames();
	}
}

