package l14_reflection;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class ReflectionExample {

	static class RentCar {

		private final String type;
		private final int rate;

		public RentCar(int length) {
			if (length < 455) {
				type = "small";
				rate = 35;
			} else if (length < 495) {
				type = "mid-sized";
				rate = 45;
			} else {
				type = "large";
				rate = 55;
			}
		}

		public void computeRentalCost(int numDays) {
			int price = numDays * rate;
			System.out.println(STR."You are renting a \{type} car for \{numDays} days");
			System.out.println(STR."The cost of your rental is $\{price}");
			System.out.println();
		}
	}

	private static void testRentCar() throws ReflectiveOperationException {

		// get constructor with specific argument
		Constructor<RentCar> constructor = RentCar.class.getConstructor(int.class);

		// initializing objects of the RentCar class
		RentCar smallCar = constructor.newInstance(340);
		RentCar midCar   = constructor.newInstance(465);
		RentCar largeCar = constructor.newInstance(545);

		// get method with specific name and parameters
		Method method = RentCar.class.getMethod("computeRentalCost", int.class);
		System.out.println(STR."Method is: \{method}");
		System.out.println();

		// call computeRentalCost method with parameter int
		method.invoke(smallCar, 10);
		method.invoke(midCar, 4);
		method.invoke(largeCar, 6);
	}

	public static void main(String[] args) {
		try {
			testRentCar();
		} catch (ReflectiveOperationException e) {
			e.printStackTrace();
		}
	}
}

