package l14_reflection;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

class VehicleSystem {

	interface IVehicle {
		void go();
	}

	static class Car implements IVehicle {
		public Car() {}
		@Override
		public void go() {
			System.out.println("Brum, brum");
		}
	}

	static class Airplane implements IVehicle {
		public Airplane() {}
		@Override
		public void go() {
			System.out.println("Bzzzzzz");
		}
	}

	static class Rocket implements IVehicle {
		public Rocket() {}
		@Override
		public void go(){
			System.out.println("BUUUUUUUUUUUM");
		}
	}

	static class NotAVehicle {
		public NotAVehicle() {}
		public void go() {
			System.out.println(":(");
		}
	}

	static class NoPublicCtor implements IVehicle {
		@Override
		public void go() {
			System.out.println("NoPublicCtor");
		}
	}

	static abstract class AbstractClass implements IVehicle {
		public AbstractClass() {}
		@Override
		public void go() {
			System.out.println("AbstractClass");
		}
	}

	public static void main(String[] args) {
		String[] names = "Car Rocket Airplane Car NotAVehicle X NoPublicCtor AbstractClass".split(" ");
		List<IVehicle> vehicles = new ArrayList<>();

		for(String name : names){
			String vehicleName = STR."\{VehicleSystem.class.getName()}$\{name}";
			try {
				IVehicle vehicle = (IVehicle) Class.forName(vehicleName).getConstructor().newInstance();
				vehicles.add(vehicle);
			} catch(ClassCastException ex) {
				System.out.println(STR."The class \{vehicleName} is not assignable to IVehicle.");
			} catch(ClassNotFoundException ex) {
				System.out.println(STR."The class \{vehicleName} has not been found.");
			} catch(NoSuchMethodException ex) {
				System.out.println(STR."The class \{vehicleName} has no default public constructor.");
			} catch(InstantiationException | IllegalAccessException | InvocationTargetException ex) {
				System.out.println(STR."The class \{vehicleName} cannot be instantiated.");
			}
		}
		System.out.println();
		vehicles.forEach(IVehicle::go);
	}
}

